==> relations-0.0.5.tar.gz: lib/view/stage_items/edge.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:vector_math/vector_math_64.dart' hide Colors;
import '../../bloc/bloc.dart';
import '../../models/graph_edge.dart';
import '../../models/graph_node.dart';
import 'stage_entity.dart';
import '../../mixin/selectable.dart';
import '../../utils/math_util.dart';

class Edge extends StageEntity {
  final GraphEdge edgeData;

  Edge({
    Key key,
    @required this.edgeData,
  }) : super(key: key);

  @override
  EdgeState createState() => EdgeState();
}

class EdgeState extends StageEntityState<Edge> with Selectable {
  final double baseStrokeWidth = 24.0;
  final double strokeWidthFactor = 4.0;

  double _strokeWidth = 20.0;
  double length;
  Matrix4 _transform;

  GraphEdge _edgeData;

  @override
  void initState() {
    super.initState();
    _edgeData = widget.edgeData;
    selected = false;

    String selectedID = stageBloc.state?.selectedEdge?.edgeData?.id ??
        GraphEdge.INVALID_ID;
    if (_edgeData.id == selectedID) {
      stageBloc.setSelectedEdge(this);
    }

    _recalculate();
  }

  GraphEdge get edgeData => _edgeData;

  void _setStrokeWidth() {
    _strokeWidth = baseStrokeWidth + (4.0 * stageInverseScale);
  }

  @override
  void updateOnScale(double newScale) {
    // updating only when scale happens
    stageScale = newScale;

    stageInverseScale = 1.0 / stageScale;

    if (stageInverseScale > 1.0) {
      _setStrokeWidth();
    }
  }

  @override
  void onStageTransformChange(Matrix4 transform) {
    if (!mounted) return;

    double newScale = transform.getMaxScaleOnAxis();
    if (stageScale != newScale) {
      startAnim(stageScale, newScale);
    }
  }

  void _onTapUp(TapUpDetails details) {
    stageBloc.setSelectedEdge(this);
  }

  Offset get centre {
    GraphNode from = graphBloc.graph.getNode(widget.edgeData.from);
    GraphNode to = graphBloc.graph.getNode(widget.edgeData.to);
    Offset centreFrom = MathUtil.centreOfNode(from);
    Offset centreTo = MathUtil.centreOfNode(to);
    return (centreFrom + centreTo) / 2.0;
  }

  void _recalculate() {
    if (!mounted) return;

    GraphNode from = graphBloc.graph.getNode(widget.edgeData.from);
    GraphNode to = graphBloc.graph.getNode(widget.edgeData.to);
    Offset centreFrom = MathUtil.centreOfNode(from);
    Offset centreTo = MathUtil.centreOfNode(to);

    final Offset toEnd = centreTo - centreFrom;
    length = toEnd.distance;
    final angle = MathUtil.angleOfVector(toEnd);

    final Offset pos = centreFrom;
    _transform = Matrix4.identity()
      ..translate(pos.dx, pos.dy - (_strokeWidth / 2.0))
      ..rotateZ(angle);
  }

  /*
   The idea is to represent a line as a Container inside Transform, so we can detect gestures on the line,
   and we can apply transformations also.
   There was an issue tough:
   Needed some fix, because if the distance between two boxes become too big,
   the ends of some edges did not touch the other box, 
   because it was shorter than the desired length
   -> width is restricted to the parent container's width.
   SOLVED by wrapping it in an OverflowBox. Altough this solution can cause performance issues.
  */
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<EdgeEvent>(
        stream: graphBloc.edgeNotifierStream
            .where((EdgeEvent event) => event.edgeID == _edgeData.id),
        builder: (BuildContext context, AsyncSnapshot<EdgeEvent> snapshot) {
          // print('Edge rebuild ${_edgeData.toString()}');
          //if (snapshot.data.type == EdgeEventType.nodeMoved) {
          _recalculate();
          //}
          return OverflowBox(
              alignment:
                  Alignment.topLeft, // must be topLeft for proper working
              minWidth: 0.0,
              minHeight: 0.0,
              maxWidth: double.infinity,
              maxHeight: double.infinity,
              child: Transform(
                  transform: _transform,
                  alignment: FractionalOffset.centerLeft,
                  child: GestureDetector(
                      onTapUp: _onTapUp,
                      child: Container(
                          // providing a bigger area invisible for gesture detection
                          width: length,
                          height: _strokeWidth,
                          padding: EdgeInsets.only(
                              top: 7,
                              bottom:
                                  7), // or: halfStrokeWidth - 2 for both top and bottom
                          color: Colors.transparent,
                          child: Container(
                            // to visualize, if selected
                            decoration: BoxDecoration(
                                color: selected
                                    ? stageBloc.theme.edgeSelectedColor
                                    : stageBloc.theme.edgeColor),
                            alignment: Alignment.center,
                            width: length,
                          )))));
        });
  }

  @override
  void dispose() {
    super.dispose();
  }
}

==> relations-1.0.0.tar.gz: lib/view/stage_items/edge.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:vector_math/vector_math_64.dart' hide Colors;
import '../../bloc/bloc.dart';
import '../../models/graph_edge.dart';
import '../../models/graph_node.dart';
import 'stage_entity.dart';
import '../../mixin/selectable.dart';
import '../../utils/math_util.dart';

class Edge extends StageEntity {
  final GraphEdge edgeData;

  Edge({
    Key key,
    @required this.edgeData,
  }) : super(key: key);

  @override
  EdgeState createState() => EdgeState();
}

class EdgeState extends StageEntityState<Edge> with Selectable {
  final double baseStrokeWidth = 24.0;
  final double strokeWidthFactor = 4.0;

  double _strokeWidth = 20.0;
  double length;
  Matrix4 _transform;

  GraphEdge _edgeData;

  @override
  void initState() {
    super.initState();
    _edgeData = widget.edgeData;
    selected = false;

    String selectedID =
        stageBloc.state?.selectedEdge?.edgeData?.id ?? GraphEdge.INVALID_ID;
    if (_edgeData.id == selectedID) {
      stageBloc.setSelectedEdge(this);
    }

    _recalculate();
  }

  GraphEdge get edgeData => _edgeData;

  void _setStrokeWidth() {
    _strokeWidth = baseStrokeWidth + (4.0 * stageInverseScale);
  }

  @override
  void updateOnScale(double newScale) {
    // updating only when scale happens
    stageScale = newScale;

    stageInverseScale = 1.0 / stageScale;

    if (stageInverseScale > 1.0) {
      _setStrokeWidth();
    }
  }

  @override
  void onStageTransformChange(Matrix4 transform) {
    if (!mounted) return;

    double newScale = transform.getMaxScaleOnAxis();
    if (stageScale != newScale) {
      startAnim(stageScale, newScale);
    }
  }

  void _onTapUp(TapUpDetails details) {
    stageBloc.setSelectedEdge(this);
  }

  Offset get centre {
    GraphNode from = graphBloc.graph.getNode(widget.edgeData.from);
    GraphNode to = graphBloc.graph.getNode(widget.edgeData.to);
    Offset centreFrom = MathUtil.centreOfNode(from);
    Offset centreTo = MathUtil.centreOfNode(to);
    return (centreFrom + centreTo) / 2.0;
  }

  void _recalculate() {
    if (!mounted) return;

    GraphNode from = graphBloc.graph.getNode(widget.edgeData.from);
    GraphNode to = graphBloc.graph.getNode(widget.edgeData.to);
    Offset centreFrom = MathUtil.centreOfNode(from);
    Offset centreTo = MathUtil.centreOfNode(to);

    final Offset toEnd = centreTo - centreFrom;
    length = toEnd.distance;
    final angle = MathUtil.angleOfVector(toEnd);

    final Offset pos = centreFrom;
    _transform = Matrix4.identity()
      ..translate(pos.dx, pos.dy - (_strokeWidth / 2.0))
      ..rotateZ(angle);
  }

  /*
   The idea is to represent a line as a Container inside Transform, so we can detect gestures on the line,
   and we can apply transformations also.
   There was an issue tough:
   Needed some fix, because if the distance between two boxes become too big,
   the ends of some edges did not touch the other box, 
   because it was shorter than the desired length
   -> width is restricted to the parent container's width.
   SOLVED by wrapping it in an OverflowBox. Altough this solution can cause performance issues.
  */
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<EdgeEvent>(
        stream: graphBloc.edgeNotifierStream
            .where((EdgeEvent event) => event.edgeID == _edgeData.id),
        builder: (BuildContext context, AsyncSnapshot<EdgeEvent> snapshot) {
          // print('Edge rebuild ${_edgeData.toString()}');
          //if (snapshot.data.type == EdgeEventType.nodeMoved) {
          _recalculate();
          //}
          return OverflowBox(
              alignment:
                  Alignment.topLeft, // must be topLeft for proper working
              minWidth: 0.0,
              minHeight: 0.0,
              maxWidth: double.infinity,
              maxHeight: double.infinity,
              child: Transform(
                  transform: _transform,
                  alignment: FractionalOffset.centerLeft,
                  child: GestureDetector(
                      onTapUp: _onTapUp,
                      child: Container(
                          // providing a bigger area invisible for gesture detection
                          width: length,
                          height: _strokeWidth,
                          padding: EdgeInsets.only(
                              top: 7,
                              bottom:
                                  7), // or: halfStrokeWidth - 2 for both top and bottom
                          color: Colors.transparent,
                          child: Container(
                            // to visualize, if selected
                            decoration: BoxDecoration(
                                color: selected
                                    ? stageBloc.theme.edgeSelectedColor
                                    : stageBloc.theme.edgeColor),
                            alignment: Alignment.center,
                            width: length,
                          )))));
        });
  }

  @override
  void dispose() {
    super.dispose();
  }
}
